require "Apollo"
require "Window"
require "CostumesLib"

local GeminiLocale = Apollo.GetPackage("Gemini:Locale-1.0").tPackage
local GeminiHook = Apollo.GetPackage("Gemini:Hook-1.0").tPackage

local Stocktaker = Apollo.GetPackage("Gemini:Addon-1.1").tPackage:NewAddon("Stocktaker", false, { "ToolTips", "SupplySatchel", "BankViewer" })

local L = GeminiLocale:GetLocale("Stocktaker", true)

local Data = {
  Types = {
    "Bags",
    "MBag",
    "Equip",
    "Bank",
    "Costumes",
    "DecorInCrate",
    "DecorPlaced",
    "AMPs",
    "Dyes",
    "Reputations",
    "Currencies"
  },
  QualityColors = {
    "ItemQuality_Inferior",
    "ItemQuality_Average",
    "ItemQuality_Good",
    "ItemQuality_Excellent",
    "ItemQuality_Superb",
    "ItemQuality_Legendary",
    "ItemQuality_Artifact"
  },
  Faction = {
    [Unit.CodeEnumFaction.ExilesPlayer] = {
      Name = Apollo.GetString("CRB_Exile"),
      Icon = "charactercreate:sprCharC_Ico_Exile_Lrg"
    },
    [Unit.CodeEnumFaction.DominionPlayer] = {
      Name = Apollo.GetString("CRB_Dominion"),
      Icon = "charactercreate:sprCharC_Ico_Dominion_Lrg"
    }
  },
  Race = {
    [GameLib.CodeEnumRace.Aurin] = {
      Name = Apollo.GetString("RaceAurin"),
      Icon = "CRB_CharacterCreateSprites:sprCC_RaceAurin"
    },
    [GameLib.CodeEnumRace.Chua] = {
      Name = Apollo.GetString("RaceChua"),
      Icon = "CRB_CharacterCreateSprites:sprCC_RaceChua"
    },
    [GameLib.CodeEnumRace.Draken] = {
      Name = Apollo.GetString("RaceDraken"),
      Icon = "CRB_CharacterCreateSprites:sprCC_RaceDraken"
    },
    [GameLib.CodeEnumRace.Granok] = {
      Name = Apollo.GetString("RaceGranok"),
      Icon = "CRB_CharacterCreateSprites:sprCC_RaceGranok"
    },
    [GameLib.CodeEnumRace.Human] = {
      Name = Apollo.GetString("RaceHuman"),
      Icon = "CRB_CharacterCreateSprites:sprCC_RaceHuman"
    },
    [GameLib.CodeEnumRace.Mechari] = {
      Name = Apollo.GetString("RaceMechari"),
      Icon = "CRB_CharacterCreateSprites:sprCC_RaceMechari"
    },
    [GameLib.CodeEnumRace.Mordesh] = {
      Name = Apollo.GetString("CRB_Mordesh"),
      Icon = "CRB_CharacterCreateSprites:sprCC_RaceMordesh"
    }
  },
  Class = {
    [GameLib.CodeEnumClass.Engineer] = {
      Name = Apollo.GetString("ClassEngineer"),
      Icon = "IconSprites:Icon_Windows_UI_CRB_Engineer"
    },
    [GameLib.CodeEnumClass.Esper] = {
      Name = Apollo.GetString("ClassESPER"),
      Icon = "IconSprites:Icon_Windows_UI_CRB_Esper"
    },
    [GameLib.CodeEnumClass.Medic] = {
      Name = Apollo.GetString("ClassMedic"),
      Icon = "IconSprites:Icon_Windows_UI_CRB_Medic"
    },
    [GameLib.CodeEnumClass.Spellslinger] = {
      Name = Apollo.GetString("ClassSpellslinger"),
      Icon = "IconSprites:Icon_Windows_UI_CRB_Spellslinger"
    },
    [GameLib.CodeEnumClass.Stalker] = {
      Name = Apollo.GetString("ClassStalker"),
      Icon = "IconSprites:Icon_Windows_UI_CRB_Stalker"
    },
    [GameLib.CodeEnumClass.Warrior] = {
      Name = Apollo.GetString("ClassWarrior"),
      Icon = "IconSprites:Icon_Windows_UI_CRB_Warrior"
    }
  },
  Path = {
    [PlayerPathLib.PlayerPathType_Explorer] = {
      Name = Apollo.GetString("PlayerPathExplorer"),
      Icon = "CRB_PlayerPathSprites:spr_Path_Explorer_Stretch"
    },
    [PlayerPathLib.PlayerPathType_Scientist] = {
      Name = Apollo.GetString("PlayerPathScientist"),
      Icon = "CRB_PlayerPathSprites:spr_Path_Scientist_Stretch"
    },
    [PlayerPathLib.PlayerPathType_Settler] = {
      Name = Apollo.GetString("PlayerPathSettler"),
      Icon = "CRB_PlayerPathSprites:spr_Path_Settler_Stretch"
    },
    [PlayerPathLib.PlayerPathType_Soldier] = {
      Name = Apollo.GetString("PlayerPathSoldier"),
      Icon = "CRB_PlayerPathSprites:spr_Path_Soldier_Stretch"
    }
  },
  Currency = {
    [Money.CodeEnumCurrencyType.CraftingVouchers] = {
      Name = "CraftingVouchers" --Apollo.GetString("CRB_Crafting_Vouchers"),
      --Description = Apollo.GetString("CRB_Crafting_Voucher_Desc")
    },
    [Money.CodeEnumCurrencyType.ElderGems] = {
      Name = "ElderGems" --Apollo.GetString("CRB_Elder_Gems"),
      --Description = Apollo.GetString("CRB_Elder_Gems_Desc")
    },
    [Money.CodeEnumCurrencyType.Prestige] = {
      Name = "Prestige" --Apollo.GetString("CRB_Prestige"),
      --Description = Apollo.GetString("CRB_Prestige_Desc")
    },
    [Money.CodeEnumCurrencyType.Renown] = {
      Name = "Renown" --Apollo.GetString("CRB_Renown"),
      --Description = Apollo.GetString("CRB_Renown_Desc")
    }
  }
}

local defaults = {
  realm = {
    Characters = {},
    Guilds = {}
  },
  char = {
    MainWindow = {
      SelectedMain = "Characters",
      SelectedCharacter = nil,
      SelectedGuild = nil,
      SelectedType = nil,
      SearchText = "",
      State = false,
      AnchorOffsets = {
        Left = -275,
        Top = -171,
        Right = 275,
        Bottom = 171
      }
    }
  }
}

function Stocktaker:OnInitialize()
  self.db = Apollo.GetPackage("Gemini:DB-1.0").tPackage:New(self, defaults)

  -- Windows
    self.xmlDoc = XmlDoc.CreateFromFile("Stocktaker.xml")
  self.xmlDoc:RegisterCallback("OnDocLoaded", self)

  -- Commands
  Apollo.RegisterSlashCommand("st", "OnST", self)
  Apollo.RegisterSlashCommand("stocktaker", "OnST", self)

  -- Events
  self:RegisterEvents()

  -- Hooks
  GeminiHook:Embed(self)

  self:PostHook(Apollo.GetAddon("ToolTips"), "CreateCallNames")

  Stocktaker.SupplySatchel = Apollo.GetAddon("SupplySatchel")
  self:RawHook(Stocktaker.SupplySatchel, "OnDocumentReady")
  self:RawHook(Stocktaker.SupplySatchel, "PopulateSatchel")

  Stocktaker.BankViewer = Apollo.GetAddon("BankViewer")
  self:RawHook(Stocktaker.BankViewer, "Initialize")
  self:RawHook(Stocktaker.BankViewer, "HideBank")
end

function Stocktaker:RegisterEvents()
  -- Money
  Apollo.RegisterEventHandler("PlayerCurrencyChanged", "OnPlayerCurrencyChanged", self)

  -- Items
  Apollo.RegisterEventHandler("ItemRemoved", "OnItemRemoved", self)
  Apollo.RegisterEventHandler("PostCommodityOrderResult", "OnPostCommodityOrderResult", self)
  Apollo.RegisterEventHandler("UpdateInventory", "OnUpdateInventory", self)

  -- Items (GuildBank)
  Apollo.RegisterEventHandler("GuildBankTab", "OnGuildBankTab", self)
  Apollo.RegisterEventHandler("GuildBankItem", "OnGuildBankItem", self)

  -- Items (Crate/Placed)
  Apollo.RegisterEventHandler("ItemSentToCrate", "OnItemSentToCrate", self)
  Apollo.RegisterEventHandler("SubZoneChanged", "OnSubZoneChanged", self)

  -- TODO: GuildBank ToolTips... MBag ToolTips if GuildBank Updated...
  -- TODO: Decor via ids...
  -- TODO: AMPs via ids...
  -- TODO: dye learnd == inventory update?
  -- TODO: dye in tooltip

  --[[
  Apollo.RegisterEventHandler("LootedItem", "OnLootedItem", self)
  Apollo.RegisterEventHandler("ItemAdded", "OnItemAdded", self)

  Apollo.RegisterEventHandler("CombatLogLoot", "OnItemLoot", self)
  Apollo.RegisterEventHandler("GenericEvent_LootChannelMessage", "OnItemLoot", self)

  Apollo.RegisterEventHandler("PostItemAuctionResult", "OnPostItemAuction", self)
  ]]--
end

function Stocktaker:OnEnable()
  while not Data.Character do
    self:SavePlayer()
  end

  self:SaveHelper()
end

-----------------------------------------------------------------------------------------------
-- Save functions
-----------------------------------------------------------------------------------------------

function Stocktaker:SavePlayer()
  if GameLib.GetPlayerUnit():IsThePlayer() then
    Data.Character = GameLib.GetPlayerUnit():GetName()
    Data.Guild = GameLib.GetPlayerUnit():GetGuildName()

    if Data.Character then
      if not self.db.realm.Characters[Data.Character] then
        self.db.realm.Characters[Data.Character] = {}
      end

      for strName, tCharacter in pairs(self.db.realm.Characters) do
        if not self.db.realm.Characters[strName].Money then self.db.realm.Characters[strName].Money = 0 end

        if not self.db.realm.Characters[strName].Level then self.db.realm.Characters[strName].Level = 0 end

        for i, strType in ipairs(Data.Types) do
          if not self.db.realm.Characters[strName][strType] then
            self.db.realm.Characters[strName][strType] = {}
          end
        end
      end

      self.db.realm.Characters[Data.Character].Faction = GameLib.GetPlayerUnit():GetFaction()
      self.db.realm.Characters[Data.Character].Race = GameLib.GetPlayerUnit():GetRaceId()
      self.db.realm.Characters[Data.Character].Class = GameLib.GetPlayerUnit():GetClassId()
      self.db.realm.Characters[Data.Character].Path = PlayerPathLib.GetPlayerPathType()
      self.db.realm.Characters[Data.Character].Zone = GetCurrentZoneName()
      self.db.realm.Characters[Data.Character].Level = GameLib.GetPlayerUnit():GetLevel()

      self.db.realm.Characters[Data.Character].Guild = Data.Guild
    end

    if Data.Guild and not self.db.realm.Guilds[Data.Guild] then
      self.db.realm.Guilds[Data.Guild] = {
        Bank = {},
        Update = "-"
      }
    end

    for i, tGuild in ipairs(GuildLib.GetGuilds()) do
      if tGuild:GetType() == GuildLib.GuildType_Guild then
        self.db.realm.Guilds[Data.Guild].Faction = GameLib.GetPlayerUnit():GetFaction()
        self.db.realm.Guilds[Data.Guild].MemberCount = tGuild:GetMemberCount()
        self.db.realm.Guilds[Data.Guild].Influence = tGuild:GetInfluence()

        break;
      end
    end
  end
end

function Stocktaker:SavePlayerCurrency()
  self.db.realm.Characters[Data.Character].Money = GameLib.GetPlayerCurrency():GetAmount()
end

function Stocktaker:SaveInventoryItems()
  local itemIdsInInventory = {}

  for i, tI in pairs(GameLib.GetPlayerUnit():GetInventoryItems()) do
    local id = "" .. tI.itemInBag:GetItemId()

    if not itemIdsInInventory[id] then
      itemIdsInInventory[id] = {
        Count = tI.itemInBag:GetStackCount(),
        Icon = tI.itemInBag:GetIcon(),
        Name = tI.itemInBag:GetName(),
        Quality = tI.itemInBag:GetItemQuality(),
        ItemId = tI.itemInBag:GetItemId()
      }
    else
      itemIdsInInventory[id].Count = itemIdsInInventory[id].Count + tI.itemInBag:GetStackCount()
    end
  end

  self.db.realm.Characters[Data.Character].Bags = itemIdsInInventory
end

function Stocktaker:SaveEquippedItems()
  local itemIdsInEquip = {}

  for i, item in ipairs(GameLib.GetPlayerUnit():GetEquippedItems()) do
    local id = "" .. item:GetItemId()

    if not itemIdsInEquip[id] then
      itemIdsInEquip[id] = {
        Count = 1,
        Icon = item:GetIcon(),
        Name = item:GetName(),
        Quality = item:GetItemQuality(),
        ItemId = item:GetItemId()
      }
    else
      itemIdsInEquip[id].Count = itemIdsInEquip[id].Count + 1
    end
  end

  self.db.realm.Characters[Data.Character].Equip = itemIdsInEquip
end

function Stocktaker:SaveBankItems()
  if Stocktaker.BankViewer.wndMain ~= nil then
    local bankBagWindow = Stocktaker.BankViewer.wndMain:FindChild("MainBagWindow")
    local maxSlots = bankBagWindow:GetTotalBagSlots()
    local itemIdsInBank = {}

    for i = 512, 512 + maxSlots do
      local item = bankBagWindow:GetItem(i)

      if item ~= nil then
        local id = "" .. item:GetItemId()

        if not itemIdsInBank[id] then
          itemIdsInBank[id] = {
            Count = item:GetStackCount(),
            Icon = item:GetIcon(),
            Name = item:GetName(),
            Quality = item:GetItemQuality(),
            ItemId = item:GetItemId()
          }
        else
          itemIdsInBank[id].Count = itemIdsInBank[id].Count + item:GetStackCount()
        end
      end
    end

    self.db.realm.Characters[Data.Character].Bank = itemIdsInBank
  end
end

function Stocktaker:SaveSupplySatchelItems()
  local itemIdsInSatchel = {}

  for category, itemList in pairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()) do
    for i, materialInfo in pairs(itemList) do
      local id = "" .. materialInfo.itemMaterial:GetItemId()

      itemIdsInSatchel[id] = {
        Count = materialInfo.nCount,
        Icon = materialInfo.itemMaterial:GetIcon(),
        Name = materialInfo.itemMaterial:GetName(),
        Quality = materialInfo.itemMaterial:GetItemQuality(),
        ItemId = materialInfo.itemMaterial:GetItemId()
      }
    end
  end

  self.db.realm.Characters[Data.Character].MBag = itemIdsInSatchel
end

function Stocktaker:SaveCostumeItems()
  local tCostumeSlots = {
    Weapon = GameLib.CodeEnumItemSlots.Weapon,
    Head = GameLib.CodeEnumItemSlots.Head,
    Shoulder = GameLib.CodeEnumItemSlots.Shoulder,
    Chest = GameLib.CodeEnumItemSlots.Chest,
    Hands = GameLib.CodeEnumItemSlots.Hands,
    Legs = GameLib.CodeEnumItemSlots.Legs,
    Feet = GameLib.CodeEnumItemSlots.Feet
  }

  local itemIdsInCostumes = {}

  for costumeId = 1, CostumesLib.GetCostumeCount() do
    local costume = CostumesLib.GetCostume(costumeId)

    for strName, slot in pairs(tCostumeSlots) do
      local item = costume:GetSlotItem(slot)

      if item then
        local id = "" .. item:GetItemId()

        if not itemIdsInCostumes[id] then
          itemIdsInCostumes[id] = {
            Count = 1,
            Icon = item:GetIcon(),
            Name = item:GetName(),
            Quality = item:GetItemQuality(),
            ItemId = item:GetItemId()
          }
        else
          itemIdsInCostumes[id].Count = itemIdsInCostumes[id].Count + 1
        end
      end
    end
  end

  self.db.realm.Characters[Data.Character].Costumes = itemIdsInCostumes
end

function Stocktaker:SaveDecorItems()
  if HousingLib.IsOnMyResidence() then
    -- Crate Decor
    local decorIdsInCrate = {}
    local residence = HousingLib.GetResidence()

    for x, tDecor in ipairs(residence:GetDecorCrateList()) do
      decorIdsInCrate["" .. tDecor.nId] = {
        Count = tDecor.nCount,
        Icon = "IconSprites:Icon_ItemMisc_Statues",
        Name = tDecor.strName,
        Quality = 2
      }
    end

    if self:hasItems(decorIdsInCrate) then
      self.db.realm.Characters[Data.Character].DecorInCrate = decorIdsInCrate
    end

    -- Placed Decor
    local decorIdsPlaced = {}

    for x, tDecor in ipairs(residence:GetPlacedDecorList()) do
      local name = tDecor:GetName()

      if not decorIdsPlaced[name] then
        decorIdsPlaced[name] = {
          Count = 1,
          Icon = "IconSprites:Icon_ItemMisc_Statues",
          Name = name,
          Quality = 2
        }
      else
        decorIdsPlaced[name].Count = decorIdsPlaced[name].Count + 1
      end
    end

    if self:hasItems(decorIdsPlaced) then
      self.db.realm.Characters[Data.Character].DecorPlaced = decorIdsPlaced
    end
  end
end

function Stocktaker:SaveAMPs()
  local augments = AbilityBook.GetEldanAugmentationData(AbilityBook.GetCurrentSpec()).tAugments
  local ampIds = {}

  for i = 1, #augments do
    if augments[i].bUnlocked then
      ampIds[augments[i].strTitle] = { --augments[i].nId
        Count = 1,
        Icon = "ClientSprites:Icon_ItemMisc_PowerCell",
        Name = augments[i].strTitle,
        Quality = 2
      }
    end
  end

  self.db.realm.Characters[Data.Character].AMPs = ampIds
end

function Stocktaker:SaveDyes()
  local dyes = {}

  for x, tDI in ipairs(GameLib.GetKnownDyes()) do
    local strName = ""

    if tDI.strName and tDI.strName:len() > 0 then
      strName = tDI.strName
    else
      strName = String_GetWeaselString(Apollo.GetString("CRB_CurlyBrackets"), "", tDI.nRampIndex)
    end

    dyes[tDI.nId] = {
      Count = 1,
      Icon = "CRB_DyeRampSprites:sprDyeRamp_" .. tDI.nRampIndex,
      Name = strName,
      Quality = 2
    }
  end

  self.db.realm.Characters[Data.Character].Dyes = dyes
end

function Stocktaker:SaveReputations()
  local reputations = {}

  for idx, tR in pairs(GameLib.GetReputationInfo()) do
    if tR.strParent == "" then
      reputations[tR.strName] = {
        Name = tR.strName,
        Level = tR.nLevel,
        Value = tR.nCurrent,
        Children = {}
      }
    end
  end

  for name, info in pairs(reputations) do
    for idx, tR in pairs(GameLib.GetReputationInfo()) do
      if name == tR.strParent then
        reputations[name].Children[tR.strName] = {
          Name = tR.strName,
          Level = tR.nLevel,
          Value = tR.nCurrent,
          Children = {}
        }
      end
    end

    for idx, tR in pairs(GameLib.GetReputationInfo()) do
      for nameC, infoC in pairs(reputations[name].Children) do
        if nameC == tR.strParent then
          reputations[name].Children[nameC].Children[tR.strName] = {
            Name = tR.strName,
            Level = tR.nLevel,
            Value = tR.nCurrent
          }
        end
      end
    end
  end

  self.db.realm.Characters[Data.Character].Reputations = reputations
end

function Stocktaker:SaveCurrencies()
  local currencies = {}

  for eType, info in pairs(Data.Currency) do
    currencies[eType] = GameLib.GetPlayerCurrency(eType):GetAmount()
  end

  self.db.realm.Characters[Data.Character].Currencies = currencies
end

-----------------------------------------------------------------------------------------------
-- Helper functions
-----------------------------------------------------------------------------------------------

function Stocktaker:SaveHelper()
  self:SavePlayerCurrency()
  self:SaveInventoryItems()
  self:SaveSupplySatchelItems()
  self:SaveEquippedItems()
  self:SaveCostumeItems()
  self:SaveDecorItems()
  self:SaveAMPs()
  self:SaveDyes()
  self:SaveReputations()
  self:SaveCurrencies()
end

function Stocktaker:MainWindowHelper()
  -- Search
  self.wndMain:FindChild("SearchContainer:Search"):SetText(self.db.char.MainWindow.SearchText)

  -- Main
  self.wndMain:FindChild("ComboBoxMain"):AddItem(L["Characters"], "", "Characters")
  self.wndMain:FindChild("ComboBoxMain"):AddItem(L["Guilds"], "", "Guilds")
  self.wndMain:FindChild("ComboBoxMain"):SelectItemByData(self.db.char.MainWindow.SelectedMain)

  self.db.char.MainWindow.SelectedMain = self.wndMain:FindChild("ComboBoxMain"):GetSelectedData()

  self:OnStocktakerMainCBChanged()

  -- Characters
  self.wndMain:FindChild("ComboBoxCharacters"):AddItem(L["All"], "", nil)

  for strName, tCharacter in pairs(self.db.realm.Characters) do
    self.wndMain:FindChild("ComboBoxCharacters"):AddItem(strName, "", strName)
  end

  if not self.db.realm.Characters[self.db.char.MainWindow.SelectedCharacter] then
    self.db.char.MainWindow.SelectedCharacter = nil
  end

  self.wndMain:FindChild("ComboBoxCharacters"):SelectItemByData(self.db.char.MainWindow.SelectedCharacter)
  self.db.char.MainWindow.SelectedCharacter = self.wndMain:FindChild("ComboBoxCharacters"):GetSelectedData()

  -- Guilds
  self.wndMain:FindChild("ComboBoxGuilds"):AddItem(L["All"], "", nil)

  for strName, tGuild in pairs(self.db.realm.Guilds) do
    self.wndMain:FindChild("ComboBoxGuilds"):AddItem(strName, "", strName)
  end

  if not self.db.realm.Guilds[self.db.char.MainWindow.SelectedGuild] then
    self.db.char.MainWindow.SelectedGuild = nil
  end

  self.wndMain:FindChild("ComboBoxGuilds"):SelectItemByData(self.db.char.MainWindow.SelectedGuild)
  self.db.char.MainWindow.SelectedGuild = self.wndMain:FindChild("ComboBoxGuilds"):GetSelectedData()

  -- Types
  self.wndMain:FindChild("ComboBoxTypes"):AddItem(L["None"], "", nil)

  for i, strType in ipairs(Data.Types) do
    self.wndMain:FindChild("ComboBoxTypes"):AddItem(L[strType], "", strType)
  end

  self.wndMain:FindChild("ComboBoxTypes"):SelectItemByData(self.db.char.MainWindow.SelectedType)
  self.db.char.MainWindow.SelectedType = self.wndMain:FindChild("ComboBoxTypes"):GetSelectedData()

  -- GType
  self.wndMain:FindChild("ComboBoxGTypes"):AddItem(L["None"], "", nil)
  self.wndMain:FindChild("ComboBoxGTypes"):AddItem(L["All"], "", "All")

  self.wndMain:FindChild("ComboBoxGTypes"):SelectItemByData(self.db.char.MainWindow.SelectedGType)
  self.db.char.MainWindow.SelectedGType = self.wndMain:FindChild("ComboBoxGTypes"):GetSelectedData()
end

function Stocktaker:IsAMP(eType)
  --[[
  394 = warrior AMP
  395 = engineer AMP
  396 = medic AMP
  397 = stalker AMP
  398 = esper AMP
  399 = spellslinger AMP
  ]]--

  return (eType >= 394 and eType <= 399)
end

function Stocktaker:SortByName(a, b)
  return a.Name < b.Name
end

function Stocktaker:hasItems(t)
  if t then
    for i, a in ipairs(t) do
      return true
    end

    for i, a in pairs(t) do
      return true
    end
  end

  return false
end

function Stocktaker:UpdateTextHelper()
  if self.db.char.MainWindow.SelectedGuild and self.db.realm.Guilds and self.db.realm.Guilds[self.db.char.MainWindow.SelectedGuild] then
    self.wndMain:FindChild("Update"):SetText(self.db.realm.Guilds[self.db.char.MainWindow.SelectedGuild].Update)
    self.wndMain:FindChild("ComboBoxGTypes"):Show(false)
  else
    self.wndMain:FindChild("Update"):SetText("")
    self.wndMain:FindChild("ComboBoxGTypes"):Show(true)
  end
end

-----------------------------------------------------------------------------------------------
-- Window functions
-----------------------------------------------------------------------------------------------

function Stocktaker:OnDocLoaded()
  if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Stocktaker", nil, self)
      self.wndMoneyTrigger = Apollo.LoadForm(self.xmlDoc, "MoneyTooltipTrigger", nil, self)
    self.wndMoneyTooltip = Apollo.LoadForm(self.xmlDoc, "MoneyTooltipFrame", nil, self)

    if self.wndMain == nil or self.wndMoneyTrigger == nil or self.wndMoneyTooltip == nil then
      Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")

      return
    end

    self.wndMain:SetAnchorOffsets(self.db.char.MainWindow.AnchorOffsets.Left, self.db.char.MainWindow.AnchorOffsets.Top, self.db.char.MainWindow.AnchorOffsets.Right, self.db.char.MainWindow.AnchorOffsets.Bottom)
    self.wndMain:Show(self.db.char.MainWindow.State)
      self.wndMoneyTrigger:Show(true)
    self.wndMoneyTooltip:Show(false)

    GeminiLocale:TranslateWindow(L, self.wndMain)

    self:MainWindowHelper()
  end
end

function Stocktaker:OnMouseEnter(wndHandler, wndControl, x, y)
  if Data.Character then
    local moneyTotal = 0
    local count = 0

    for strName, tCharacter in pairs(self.db.realm.Characters) do
      count = count + 1

      local newEntry = Apollo.LoadForm(self.xmlDoc, "MoneyTooltipEntry", self.wndMoneyTooltip, self)
      newEntry:FindChild("Name"):SetText(strName)
      newEntry:FindChild("CurrencyDisplay"):SetAmount(tCharacter.Money)

      newEntry:SetAnchorOffsets(0, 15 * (count - 1), 260, 15 * count + 15)

      moneyTotal = moneyTotal + tCharacter.Money
    end

    local newEntry = Apollo.LoadForm(self.xmlDoc, "MoneyTooltipEntry", self.wndMoneyTooltip, self)
    newEntry:FindChild("Name"):SetText(L["Total"])
    newEntry:FindChild("CurrencyDisplay"):SetAmount(moneyTotal)

    newEntry:SetAnchorOffsets(0, 15 * count + 5, 260, 15 * (count + 1) + 15)

    local eLeft, eTop, eRight, eBottom = self.wndMoneyTooltip:GetAnchorOffsets()

    self.wndMoneyTooltip:SetAnchorOffsets(eLeft, eBottom - (15 * (count + 2) + 15), eRight, eBottom)

    self.wndMoneyTooltip:Show(true)
  end
end

function Stocktaker:OnMouseExit(wndHandler, wndControl, x, y)
  self.wndMoneyTooltip:Show(false)
  self.wndMoneyTooltip:DestroyChildren()
end

function Stocktaker:OnClick(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
  self.db.char.MainWindow.State = not self.db.char.MainWindow.State

  if self.db.char.MainWindow.State then
    self:RedrawList()
  end

  self.wndMain:Show(self.db.char.MainWindow.State)
end

-- Main Window

function Stocktaker:OnGenerateTooltip(wndHandler, wndControl)
  if wndControl ~= wndHandler then return end

  --wndControl:SetTooltipDoc(nil)

  local item = wndControl:GetData()

  if item ~= nil then
    local equipped = item:GetEquippedItemForItemType()

    Tooltip.GetItemTooltipForm(self, wndControl, item, {bPrimary = true, bSelling = false, itemCompare = equipped})
  end
end

function Stocktaker:OnExitTooltip(wndHandler, wndControl, x, y)
  wndControl:SetTooltipDoc(nil)
end

function Stocktaker:OnStocktakerMainCBChanged(wndHandler, wndControl)
  self.db.char.MainWindow.SelectedMain = self.wndMain:FindChild("ComboBoxMain"):GetSelectedData()

  if self.db.char.MainWindow.SelectedMain == "Characters" then
    self.wndMain:FindChild("ComboBoxCharacters"):Show(true)
    self.wndMain:FindChild("ComboBoxGuilds"):Show(false)
    self.wndMain:FindChild("ComboBoxTypes"):Show(true)
    self.wndMain:FindChild("Update"):Show(false)
    self.wndMain:FindChild("ComboBoxGTypes"):Show(false)
  else
    self.wndMain:FindChild("ComboBoxCharacters"):Show(false)
    self.wndMain:FindChild("ComboBoxGuilds"):Show(true)
    self.wndMain:FindChild("ComboBoxTypes"):Show(false)
    self.wndMain:FindChild("Update"):Show(true)

    self:UpdateTextHelper()
  end

  self:RedrawList()
end

function Stocktaker:OnStocktakerCharacterCBChanged(wndHandler, wndControl)
  self.db.char.MainWindow.SelectedCharacter = self.wndMain:FindChild("ComboBoxCharacters"):GetSelectedData()
  self:RedrawList()
end

function Stocktaker:OnStocktakerGuildsCBChanged(wndHandler, wndControl)
  self.db.char.MainWindow.SelectedGuild = self.wndMain:FindChild("ComboBoxGuilds"):GetSelectedData()

  self:UpdateTextHelper()

  self:RedrawList()
end

function Stocktaker:OnStocktakerTypesCBChanged(wndHandler, wndControl)
  self.db.char.MainWindow.SelectedType = self.wndMain:FindChild("ComboBoxTypes"):GetSelectedData()
  self:RedrawList()
end

function Stocktaker:OnStocktakerGTypesCBChanged(wndHandler, wndControl)
  self.db.char.MainWindow.SelectedGType = self.wndMain:FindChild("ComboBoxGTypes"):GetSelectedData()
  self:RedrawList()
end

function Stocktaker:OnStocktakerButtonClose(wndHandler, wndControl)
  self.db.char.MainWindow.State = false
  self.wndMain:Show(false)
  self.wndMain:FindChild("ListContainer"):DestroyChildren()
end

function Stocktaker:OnStocktakerEntryButtonDelete(wndHandler, wndControl)
  self.db.realm.Characters[wndControl:GetData()] = nil

  --self:MainWindowHelper() -- TODO: delete char from combobox

  self:RedrawList()
end

function Stocktaker:OnStocktakerGEntryButtonDelete(wndHandler, wndControl)
  self.db.realm.Guilds[wndControl:GetData()] = nil

  --self:MainWindowHelper() -- TODO: delete guild from combobox

  self:RedrawList()
end

function Stocktaker:RedrawList()
  local listContainer = self.wndMain:FindChild("ListContainer")

  listContainer:DestroyChildren()

  Data.ListLength = 0

  -- if self.db.char.MainWindow.SelectedType == "Bank" and self.db.char.MainWindow.SelectedMain == "Characters" then
  --   local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerErrorEntry", listContainer)

  --   newEntry:FindChild("Error"):SetText(L["Error_Coming_Soon"])

  --   Data.ListLength = 1
  -- else
  if self.db.char.MainWindow.SelectedType == "Reputations" and self.db.char.MainWindow.SelectedMain == "Characters" then
    if not self.db.char.MainWindow.SelectedCharacter then
      local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerErrorEntry", listContainer)

      newEntry:FindChild("Error"):SetText(L["Error_Choose_Character"])

      Data.ListLength = 1
    else
      self:RedrawReputations(self.db.realm.Characters[self.db.char.MainWindow.SelectedCharacter].Reputations, listContainer, {"", 0})
    end
  elseif self.db.char.MainWindow.SelectedType == "Currencies" and self.db.char.MainWindow.SelectedMain == "Characters" then
    local list = self.db.realm.Characters

    if self.db.char.MainWindow.SelectedCharacter then
      list = {
        [self.db.char.MainWindow.SelectedCharacter] = self.db.realm.Characters[self.db.char.MainWindow.SelectedCharacter]
      }
    end

    for strName, tCharacter in pairs(list) do
      local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerCurrencyEntry", listContainer)

      newEntry:FindChild("Name"):SetText(strName)

      for eType, tData in pairs(Data.Currency) do
        if tCharacter.Currencies[eType] and tCharacter.Currencies[eType] > 0 then
          newEntry:FindChild(tData.Name):SetMoneySystem(eType)
          newEntry:FindChild(tData.Name):SetAmount(tCharacter.Currencies[eType])
        else
          newEntry:FindChild(tData.Name):Show(false)
        end
      end

      newEntry:FindChild("Money"):SetAmount(tCharacter.Money)

      Data.ListLength = Data.ListLength + 1
    end
  elseif not self.db.char.MainWindow.SelectedCharacter and self.db.char.MainWindow.SelectedMain == "Characters" then
    if not self.db.char.MainWindow.SelectedType then -- All Characters no Type
      for strName, tCharacter in pairs(self.db.realm.Characters) do
        local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerCharacterListEntry", listContainer)

        newEntry:FindChild("IconFaction"):SetSprite(tCharacter.Faction and Data.Faction[tCharacter.Faction].Icon or "")
        newEntry:FindChild("IconRace"):SetSprite(tCharacter.Race and Data.Race[tCharacter.Race].Icon or "")
        newEntry:FindChild("IconClass"):SetSprite(tCharacter.Class and Data.Class[tCharacter.Class].Icon or "")
        newEntry:FindChild("IconPath"):SetSprite(tCharacter.Path and Data.Path[tCharacter.Path].Icon or "")

        local nameLine = strName .. " (" .. (tCharacter.Level > 0 and tCharacter.Level or "n/a") .. ")"

        if tCharacter.Guild then
          nameLine = nameLine .. " <" .. tCharacter.Guild .. ">"
        end

        if tCharacter.Zone then
          nameLine = nameLine .. " - " .. tCharacter.Zone
        end

        newEntry:FindChild("Name"):SetText(nameLine)
        newEntry:FindChild("ButtonDelete"):SetData(strName)

        Data.ListLength = Data.ListLength + 1
      end
    else -- All Characters 1 Type
      local newList = self:BuildList(self.db.realm.Characters)

      self:RedrawEntrys(newList, listContainer)
    end
  elseif self.db.char.MainWindow.SelectedType and self.db.char.MainWindow.SelectedMain == "Characters"  then -- 1 Character 1 Type
    self:RedrawEntrys(self.db.realm.Characters[self.db.char.MainWindow.SelectedCharacter][self.db.char.MainWindow.SelectedType], listContainer)
  elseif self.db.char.MainWindow.SelectedMain == "Guilds" then
    if not self.db.char.MainWindow.SelectedGuild then
      if not self.db.char.MainWindow.SelectedGType then
        for strName, tGuild in pairs(self.db.realm.Guilds) do
          local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerGuildListEntry", listContainer)

          newEntry:FindChild("IconFaction"):SetSprite(Data.Faction[tGuild.Faction].Icon)

          local name = "<" .. strName .. "> - "
          name = name .. Apollo.GetString("Guild_MemberCountLabel") .. ": " .. tGuild.MemberCount .. " - "
          name = name .. Apollo.GetString("GuildPerks_InfluenceLabel") .. " " .. tGuild.Influence

          newEntry:FindChild("Name"):SetText(name)
          newEntry:FindChild("ButtonDelete"):SetData(strName)

          Data.ListLength = Data.ListLength + 1
        end
      else
        local newList = self:BuildList(self.db.realm.Guilds)

        self:RedrawEntrys(newList, listContainer)
      end
    else
      self:RedrawEntrys(self.db.realm.Guilds[self.db.char.MainWindow.SelectedGuild].Bank, listContainer)
    end
  end

  if Data.ListLength == 0 then
    local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerErrorEntry", listContainer)

    newEntry:FindChild("Error"):SetText(L["None"])

    Data.ListLength = 1
  end

  listContainer:ArrangeChildrenVert(0, SortChildren)
  listContainer:SetVScrollPos(0)
end

function SortChildren(a, b)
  local sType = "Name"

  if a:FindChild("Sort") then
    sType = "Sort"
  end

  return (a:FindChild(sType):GetText() < b:FindChild(sType):GetText())
end

function Stocktaker:BuildList(list)
  local retList = {}

  for strName, tList in pairs(list) do
    local tPairs = (self.db.char.MainWindow.SelectedMain == "Characters" and tList[self.db.char.MainWindow.SelectedType] or tList.Bank)

    for itemId, itemData in pairs(tPairs) do
      if not retList[itemId] then
        retList[itemId] = {
          Count = itemData.Count,
          Icon = itemData.Icon,
          Name = itemData.Name,
          Quality = itemData.Quality
        }

        if itemData.ItemId then
          retList[itemId].ItemId = itemData.ItemId
        end
      else
        retList[itemId].Count = retList[itemId].Count + itemData.Count
      end
    end
  end

  return retList
end

function Stocktaker:RedrawEntrys(list, listContainer)
  table.sort(list, SortByName)

  for itemId, itemData in pairs(list) do
    if not self.db.char.MainWindow.SearchText or itemData.Name:lower():gsub("–", "-"):match(self.db.char.MainWindow.SearchText) then -- btw '–' is not '-', the 1st one is longer! http://en.wikipedia.org/wiki/Dash
      local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerListEntry", listContainer)

      newEntry:FindChild("Icon"):SetSprite(itemData.Icon)
      newEntry:FindChild("Name"):SetText(itemData.Name)
      newEntry:FindChild("Name"):SetTextColor(Data.QualityColors[itemData.Quality])
      newEntry:FindChild("Data"):SetText(itemData.Count)

      if itemData.ItemId then
        local item = Item.GetDataFromId(itemData.ItemId)
        newEntry:SetData(item)
      end

      Data.ListLength = Data.ListLength + 1
    end
  end
end

function Stocktaker:RedrawReputations(list, listContainer, data)
  for name, info in pairs(list) do
    if self:hasItems(info.Children) or (info.Value and info.Value > 0) then
      local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerRepListEntry", listContainer)

      local space = ""
      local parent = ""

      for i = 1, data[2] do
        space = space .. "    "
      end

      newEntry:FindChild("Name"):SetText(space .. info.Name)

      if info.Value and info.Value > 0 then
        local tLevel = GameLib.GetReputationLevels()

        newEntry:FindChild("BarContainer:ProgressBar"):SetMax(tLevel[info.Level].nMax)
        newEntry:FindChild("BarContainer:ProgressBar"):SetProgress(info.Value)
        newEntry:FindChild("BarContainer:RepLevel"):SetText(tLevel[info.Level].strName)
        newEntry:FindChild("Data"):SetText("" .. info.Value .. "/" .. tLevel[info.Level].nMax)
      else
        newEntry:FindChild("BarContainer"):Show(false)
        newEntry:FindChild("Data"):Show(false)
      end

      if data[1] ~= "" then
        parent = data[1] .. "->"
      end

      newEntry:FindChild("Sort"):SetText(parent .. info.Name)

      Data.ListLength = Data.ListLength + 1

      if self:hasItems(info.Children) then
        self:RedrawReputations(info.Children, listContainer, {parent .. info.Name, data[2]+1})
      end
    end
  end
end

function Stocktaker:OnWindowClosed()
  self.db.char.MainWindow.State = false
end

function Stocktaker:OnWindowSizeChanged()
  local mLeft, mTop, mRight, mBottom = self.wndMain:GetAnchorOffsets()

  local ancr = self.db.char.MainWindow.AnchorOffsets

  --[[ -- Too many calls!!!
  if mRight - mLeft < 534 then
    mLeft = ancr.Left
    mRight = ancr.Right

    self.wndMain:SetAnchorOffsets(mLeft, mTop, mRight, mBottom)
  end

  if mBottom - mTop < 326 then
    mTop = ancr.Top
    mBottom = ancr.Bottom

    self.wndMain:SetAnchorOffsets(mLeft, mTop, mRight, mBottom)
  end
  ]]--

  self.db.char.MainWindow.AnchorOffsets = {
    Left = mLeft,
    Top = mTop,
    Right = mRight,
    Bottom = mBottom
  }
end

function Stocktaker:OnWindowMove()
  self:OnWindowSizeChanged()
end

-----------------------------------------------------------------------------------------------
-- Event Callbacks
-----------------------------------------------------------------------------------------------

function Stocktaker:OnST(strCmd, strArg)
  if strArg == "" then
    self:RedrawList()

    self.db.char.MainWindow.State = true
    self.wndMain:Show(true)
  elseif strArg == "resetposition" or strArg == "rp" then
    self.db.char.MainWindow.AnchorOffsets = defaults.char.MainWindow.AnchorOffsets

    local a = self.db.char.MainWindow.AnchorOffsets

    self.wndMain:SetAnchorOffsets(a.Left, a.Top, a.Right, a.Bottom)
  end
end

function Stocktaker:OnPlayerCurrencyChanged()
  self:SavePlayerCurrency()
  self:SaveCurrencies()
end

function Stocktaker:OnItemRemoved(item, count)
  Data.ReloadTooltipId = item:GetItemId()
end

function Stocktaker:OnPostCommodityOrderResult(eAuctionPostResult, orderSource, nActualCost)
  if not orderSource:IsBuy() then
    Data.ReloadTooltipId = orderSource:GetItem():GetItemId()
  end
end

function Stocktaker:OnUpdateInventory()
  self:SaveHelper()

  if Data.ReloadTooltipId then
    Event_FireGenericEvent("UpdateSupplySatchelTooltips", false)
  end

  self:SaveBankItems()
end

function Stocktaker:OnItemSentToCrate(item)
  local id = "" .. item:GetHousingDecorInfoId()

  if not self.db.realm.Characters[Data.Character].DecorInCrate[id] then
    self.db.realm.Characters[Data.Character].DecorInCrate[id] = {
      Count = 1,
      Icon = "IconSprites:Icon_ItemMisc_Statues",
      Name = item:GetName(),
      Quality = 2
    }
  else
    self.db.realm.Characters[Data.Character].DecorInCrate[id].Count = self.db.realm.Characters[Data.Character].DecorInCrate[id].Count + 1
  end
end

function Stocktaker:OnSubZoneChanged(id)
  self.db.realm.Characters[Data.Character].Zone = GetCurrentZoneName()

  self:SaveDecorItems()
end

-- Helper
function Stocktaker:GetDateTime()
  local tTime = GameLib.GetLocalTime()
  local strTime = tTime.nHour .. ":" .. tTime.nMinute .. ":" .. tTime.nSecond
  return strTime .. " " .. tTime.nDay .. "." .. tTime.nMonth .. "." .. tTime.nYear
end

-- GuildBank
function Stocktaker:OnGuildBankTab(guildOwner, nTab)
  local gbank = {}

  for i = 1, guildOwner:GetBankTabCount() do
    for idx, tC in ipairs(guildOwner:GetBankTab(i)) do
      local id = "" .. tC.itemInSlot:GetItemId()

      if not gbank[id] then
        gbank[id] = {
          Count = tC.itemInSlot:GetStackCount(),
          Icon = tC.itemInSlot:GetIcon(),
          Name = tC.itemInSlot:GetName(),
          Quality = tC.itemInSlot:GetItemQuality(),
          ItemId = tC.itemInSlot:GetItemId()
        }
      else
        gbank[id].Count = gbank[id].Count + tC.itemInSlot:GetStackCount()
      end
    end
  end

  self.db.realm.Guilds[Data.Guild].Bank = gbank
  self.db.realm.Guilds[Data.Guild].Update = self:GetDateTime()
end

function Stocktaker:OnGuildBankItem(guildOwner, nTab, nInventorySlot, itemUpdated, bRemoved)
  local id = "" .. itemUpdated:GetItemId()

  if bRemoved then
    self.db.realm.Guilds[Data.Guild].Bank[id].Count = self.db.realm.Guilds[Data.Guild].Bank[id].Count - itemUpdated:GetStackCount()

    if self.db.realm.Guilds[Data.Guild].Bank[id].Count < 1 then
      self.db.realm.Guilds[Data.Guild].Bank[id] = nil
    end
  else
    if not self.db.realm.Guilds[Data.Guild].Bank[id] then
      self.db.realm.Guilds[Data.Guild].Bank[id] = {
        Count = itemUpdated:GetStackCount(),
        Icon = itemUpdated:GetIcon(),
        Name = itemUpdated:GetName(),
        Quality = itemUpdated:GetItemQuality(),
        ItemId = itemUpdated:GetItemId()
      }
    else
      self.db.realm.Guilds[Data.Guild].Bank[id].Count = self.db.realm.Guilds[Data.Guild].Bank[id].Count + itemUpdated:GetStackCount()
    end
  end

  --Event_FireGenericEvent("GuildBankTab", guildOwner, nTab)

  self.db.realm.Guilds[Data.Guild].Update = self:GetDateTime()
end

function Stocktaker:OnSearchTextChanged(wndHandler, wndControl, strText)
  self.db.char.MainWindow.SearchText = strText:gsub("[%-%.%+%[%]%(%)%$%^%%%?%*]", "%%%1"):gsub("%z", "%%z"):lower()

  self:RedrawList()
end

-----------------------------------------------------------------------------------------------
-- Hooked functions
-----------------------------------------------------------------------------------------------

-- ToolTips
function Stocktaker:CreateCallNames(luaCaller)
  if Data.Character then
    local origItemTooltip = Tooltip.GetItemTooltipForm

    Tooltip.GetItemTooltipForm = function(luaCaller, wndControl, item, bStuff, nCount)
      wndControl:SetTooltipDoc(nil)

      local wndTooltip, wndTooltipComp = origItemTooltip(luaCaller, wndControl, item, bStuff, nCount)

      if wndTooltip then
        local itemId = "" .. item:GetItemId()

        if itemId then
          local count = 0

          -- Chars
          -----------------------------------------------------------------------------------------------
          local tooltipInfo = Apollo.LoadForm(self.xmlDoc, "StocktakerTooltipInfo", wndTooltip, self)
          local listContainer = tooltipInfo:FindChild("ListContainer")

          local nType = item:GetItemType()
          local IId = item:GetHousingDecorInfoId()
          local SName = nil

          if item:IsEquippable() then
            tooltipInfo:FindChild("Equip"):Show(true)
          elseif IId > 0 then
            tooltipInfo:FindChild("Crate"):Show(true)
          elseif self:IsAMP(nType) then
            SName = item:GetDetailedInfo().tPrimary.arSpells[1].strName

            tooltipInfo:FindChild("AMPs"):Show(true)
          elseif nType == 332 then -- dye
            tooltipInfo:FindChild("Crate"):Show(true)
          else
            tooltipInfo:FindChild("MBag"):Show(true)
          end

          local bagsTotal = 0
          local mbagTotal = 0
          local equipTotal = 0
          local bankTotal = 0

          for strCharacterName, tCharacter in pairs(self.db.realm.Characters) do
            count = count + 1

            local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerTooltipEntry", listContainer)
            newEntry:SetAnchorOffsets(0, 15 * (count - 1), 300, 15 * count + 10)

            local bagsVal = (tCharacter.Bags[itemId] and tCharacter.Bags[itemId].Count or 0)

            local eqVal = 0

            if item:IsEquippable() then
              eqVal = (tCharacter.Equip[itemId] and tCharacter.Equip[itemId].Count or 0)
              eqVal = eqVal + (tCharacter.Costumes[itemId] and tCharacter.Costumes[itemId].Count or 0)
            elseif IId > 0 then
              eqVal = (tCharacter.DecorInCrate["" .. IId] and tCharacter.DecorInCrate["" .. IId].Count or 0)
              eqVal = eqVal + (tCharacter.DecorPlaced[item:GetName()] and tCharacter.DecorPlaced[item:GetName()].Count or 0)
            elseif SName then
              eqVal = (tCharacter.AMPs[SName] and tCharacter.AMPs[SName].Count or 0)
            --elseif nType == 332 then -- dye
            else
              eqVal = (tCharacter.MBag[itemId] and tCharacter.MBag[itemId].Count or 0)
            end

            local bankVal = (tCharacter.Bank[itemId] and tCharacter.Bank[itemId].Count or 0)

            newEntry:FindChild("Name"):SetText(strCharacterName)
            newEntry:FindChild("Bags"):SetText(bagsVal)
            newEntry:FindChild("Equip"):SetText(eqVal)
            newEntry:FindChild("Bank"):SetText(bankVal)

            bagsTotal = bagsTotal + bagsVal
            equipTotal = equipTotal + eqVal
            bankTotal = bankTotal + bankVal
          end

          local total = bagsTotal + equipTotal + bankTotal

          local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerTooltipEntry", listContainer)
          newEntry:SetAnchorOffsets(0, 15 * count + 5, 300, 15 * (count + 1) + 10)
          newEntry:FindChild("Name"):SetText(L["Total"] .. " (" .. total .. ")")
          newEntry:FindChild("Bags"):SetText(bagsTotal)
          newEntry:FindChild("Equip"):SetText(equipTotal)
          newEntry:FindChild("Bank"):SetText(bankTotal)

          GeminiLocale:TranslateWindow(L, tooltipInfo)

          local eLeft, eTop, eRight, eBottom = tooltipInfo:GetAnchorOffsets()
          local nLeft, nTop, nRight, nBottom = wndTooltip:GetAnchorOffsets()

          eBottom = 0

          if total > 0 then
            eBottom = 60 + 15 * (count + 1)
            nBottom = nBottom - 30

            listContainer:SetAnchorOffsets(nLeft, 35, nRight, eBottom)
          end

          tooltipInfo:SetAnchorOffsets(nLeft, nBottom, nRight, nBottom + eBottom)

          -- Guilds
          -----------------------------------------------------------------------------------------------
          local tooltipInfoG = Apollo.LoadForm(self.xmlDoc, "StocktakerTooltipGuildInfo", wndTooltip, self)
          local listContainerG = tooltipInfoG:FindChild("ListContainer")

          local totalG = 0

          count = 0

          for strGuildName, tGuild in pairs(self.db.realm.Guilds) do
            count = count + 1

            local gVal = (tGuild.Bank[itemId] and tGuild.Bank[itemId].Count or 0)

            local newEntry = Apollo.LoadForm(self.xmlDoc, "StocktakerTooltipGuildEntry", listContainerG)

            newEntry:SetAnchorOffsets(0, 15 * (count - 1), 300, 15 * count + 10)
            newEntry:FindChild("Name"):SetText(strGuildName)
            newEntry:FindChild("GBank"):SetText(gVal)
            newEntry:FindChild("Update"):SetText(tGuild.Update)

            totalG = totalG + gVal
          end

          GeminiLocale:TranslateWindow(L, tooltipInfoG)

          local xLeft, xTop, xRight, xBottom = tooltipInfoG:GetAnchorOffsets()

          xBottom = 0

          local eBottomOff = 0

          if totalG > 0 then
            xBottom = 55 + 15 * count

            if total < 1 then
              nBottom = nBottom - 30
            else
              eBottomOff = 30
            end

            listContainerG:SetAnchorOffsets(xLeft, 35, xRight, xBottom)
          end

          tooltipInfoG:SetAnchorOffsets(xLeft, nBottom + eBottom - eBottomOff, xRight, nBottom + eBottom + xBottom - eBottomOff)

          wndTooltip:SetAnchorOffsets(nLeft, nTop, nRight, nBottom + eBottom + xBottom - eBottomOff)
        end
      end

      return wndTooltip, wndTooltipComp
    end
  end
end

-- SupplySatchel ToolTips
function Stocktaker:OnDocumentReady(luaCaller)
  self.hooks[Stocktaker.SupplySatchel].OnDocumentReady(luaCaller)

  Apollo.RegisterEventHandler("UpdateSupplySatchelTooltips", "PopulateSatchel", Stocktaker.SupplySatchel)
end

function Stocktaker:PopulateSatchel(luaCaller, bRescroll)
  if Stocktaker.SupplySatchel.tItemCache then
    if Data.ReloadTooltipId then
      for strCategory, arItems in pairs(GameLib.GetPlayerUnit():GetSupplySatchelItems(0)) do
        local tCacheCategory = Stocktaker.SupplySatchel.tItemCache[strCategory]
        if tCacheCategory then
          for idx, tCurrItem in ipairs(arItems) do
            if tCurrItem.itemMaterial:GetItemId() == Data.ReloadTooltipId then
              Tooltip.GetItemTooltipForm(Stocktaker.SupplySatchel, tCacheCategory.arItems[idx].wndItem, tCurrItem.itemMaterial, {bPrimary = true, bSelling = false}, tCurrItem.nCount)
            end
          end
        end
      end

      Data.ReloadTooltipId = nil
    else
      self.hooks[Stocktaker.SupplySatchel].PopulateSatchel(luaCaller, bRescroll)
    end
  end
end

function Stocktaker:Initialize(luaCaller)
  self.hooks[Stocktaker.BankViewer].Initialize(luaCaller)
  self:SaveBankItems()
end

function Stocktaker:HideBank(luaCaller)
  self:SaveBankItems()
  self.hooks[Stocktaker.BankViewer].HideBank(luaCaller)
end
